import 'dart:async';

import 'package:flutter/services.dart';

class AppierFlutter {
  static const MethodChannel _channel = const MethodChannel('appier_flutter');

  static Future<void> configure(String appId,
      {String senderId, String appGroup, bool isDev = false}) async {
    if (appId == null || appId.trim().isEmpty) {
      print("APPIER: Invalid AppId passed");
      return;
    }

    isConfigured = true;

    await _channel.invokeMethod('configure', {
      'appId': appId,
      'senderId': senderId,
      'appGroup': appGroup,
      'isDev': isDev
    });
  }

  static Future<void> setUniversalLinkDomains(List<String> linkDomains) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel
        .invokeMethod('setUniversalLinkDomains', {'linkDomains': linkDomains});
  }

  static Future<void> logEvent(String eventName,
      {Map<String, dynamic> parameters, double vts, String vtsCurr}) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }

    if (eventName == null || eventName.trim().isEmpty) {
      print("APPIER: Invalid Event Name passed");
      return;
    }

    await _channel.invokeMethod('logEvent', {
      'eventName': eventName,
      'parameters': parameters,
      'vts': vts,
      'vtsCurr': vtsCurr
    });
  }

  static Future<void> flush() async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('flush', {});
  }

  static Future<void> setUserId(String userId) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setUserId', {'userId': userId});
  }

  static Future<void> setName(String name) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setName', {'name': name});
  }

  static Future<void> setFirstName(String firstName) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setFirstName', {'firstName': firstName});
  }

  static Future<void> setLastName(String lastName) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setLastName', {'lastName': lastName});
  }

  static Future<void> setCity(String city) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setCity', {'city': city});
  }

  static Future<void> setEmail(String email) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setEmail', {'email': email});
  }

  static Future<void> setPhoneNumber(String phoneNo) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setPhoneNumber', {'phoneNo': phoneNo});
  }

  static Future<void> setDayOfBirth(int day) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setDayOfBirth', {'day': day});
  }

  static Future<void> setMonthOfBirth(int month) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setMonthOfBirth', {'month': month});
  }

  static Future<void> setYearOfBirth(int year) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setYearOfBirth', {'year': year});
  }

  static Future<void> setCustomKey(String key, dynamic value) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }

    if (key == null || key.trim().isEmpty) {
      print("APPIER: Invalid key passed");
      return;
    }

    if (value is String ||
        value is double ||
        value is int ||
        value is num ||
        value is bool) {
      await _channel.invokeMethod('setCustomKey', {'key': key, 'value': value});
    } else {
      print(
          "APPIER: value must be string, number(num or int or double) or boolean");
    }
  }

  static Future<void> setAttributionWindow(int seconds) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setAttributionWindow', {'seconds': seconds});
  }

  static Future<void> setClickAttributionWindow(int seconds) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel
        .invokeMethod('setClickAttributionWindow', {'seconds': seconds});
  }
}

bool isConfigured = false;
